import { L10n } from '@syncfusion/ej2-base';
import { DocumentEditor } from '../../document-editor/index';
import { MenuEventArgs } from '@syncfusion/ej2-splitbuttons';
import { ItemModel } from '@syncfusion/ej2-splitbuttons';
/**
 * Helper class for line spacing operations in Document Editor
 */
export declare class LineSpacingHelper {
    /**
     * Get line spacing items for dropdown
     *
     * @param {L10n} localObj - Localization object
     * @returns {ItemModel[]} Array of line spacing items
     */
    static getLineSpacingItems(localObj: L10n): ItemModel[];
    /**
     * Apply line spacing to the document
     *
     * @param {DocumentEditor} documentEditor - Document editor instance
     * @param {string} text - Selected line spacing text
     * @param {Object} appliedLineSpacing - Reference to store the applied line spacing
     * @param {string} appliedLineSpacing.value - The value to store the applied line spacing
     * @param {L10n} localObj - Localization object
     * @returns {void}
     */
    static applyLineSpacing(documentEditor: DocumentEditor, text: string, appliedLineSpacing: {
        value: string;
    }, localObj: L10n): void;
    /**
     * Get the current line spacing value from document
     *
     * @param {DocumentEditor} documentEditor - Document editor instance
     * @param {L10n} localObj - Localization object
     * @returns {string} The current line spacing text
     */
    static getCurrentLineSpacing(documentEditor: DocumentEditor, localObj: L10n): string;
    /**
     * Customize the line spacing dropdown item rendering
     *
     * @param {MenuEventArgs} args - Menu event arguments
     * @param {string} appliedLineSpacing - Currently applied line spacing
     * @returns {void}
     */
    static customizeLineSpacingItem(args: MenuEventArgs, appliedLineSpacing: string): void;
}
