import { DocumentEditorContainer } from '../../document-editor-container';
import { RibbonGroupModel } from '@syncfusion/ej2-ribbon';
/**
 * Constants for control group identification
 */
export declare const CONTROL_GROUP: string;
export declare const CONTENT_CONTROL_ID: string;
export declare const RICHTEXT_CONTENT_CONTROL_ID: string;
export declare const PLAINTEXT_CONTENT_CONTROL_ID: string;
export declare const COMBOBOX_CONTENT_CONTROL_ID: string;
export declare const DROPDOWNDOWN_CONTENT_CONTROL_ID: string;
export declare const DATEPICKER_CONTENT_CONTROL_ID: string;
export declare const CHECKBOX_CONTENT_CONTROL_ID: string;
export declare const PICTURE_CONTENT_CONTROL_ID: string;
/**
 * ControlGroup module
 * @private
 */
export declare class ControlGroup {
    private container;
    private ribbonId;
    private contentControlDropDown;
    /**
     * Constructor for ControlGroup class
     * @param {DocumentEditorContainer} container - DocumentEditorContainer instance
     */
    constructor(container: DocumentEditorContainer);
    /**
     * Get the DocumentEditor instance
     * @returns {DocumentEditor} The DocumentEditor instance
     */
    private readonly documentEditor;
    /**
     * Get the Control group model
     * @returns {RibbonGroupModel} The Control group model
     */
    getGroupModel(): RibbonGroupModel;
    /**
     * Get the Content Control dropdown model
     * @returns {RibbonItemModel} The Content Control dropdown model
     */
    private getContentControlDropDownModel;
    /**
     * Handle content control dropdown selection
     * @param {MenuEventArgs} args - Menu event arguments
     * @returns {void}
     */
    private onContentControlDropDownSelect;
    /**
     * Update UI based on current selection
     * @returns {void}
     * @private
     */
    updateSelection(): void;
    /**
     * Destroy the ControlGroup instance
     * @returns {void}
     */
    destroy(): void;
}
