import { DocumentEditorContainer } from '../../document-editor-container';
import { RibbonGroupModel } from '@syncfusion/ej2-ribbon';
/**
 * Constants for protect group identification
 */
export declare const PROTECT_GROUP: string;
export declare const RESTRICT_EDITING_ID: string;
/**
 * ProtectGroup module
 */
export declare class ProtectGroup {
    private container;
    private ribbonId;
    /**
     * Constructor for ProtectGroup class
     * @param {DocumentEditorContainer} container - DocumentEditorContainer instance
     */
    constructor(container: DocumentEditorContainer);
    /**
     * Get the DocumentEditor instance
     * @returns {DocumentEditor} The document editor instance
     */
    private readonly documentEditor;
    /**
     * Get the Protect group model
     * @returns {RibbonGroupModel} The ribbon group model
     */
    getGroupModel(): RibbonGroupModel;
    /**
     * Get the Restrict Editing button model
     * @returns {RibbonItemModel} The ribbon item model
     */
    private getRestrictEditingButtonModel;
    /**
     * Handle restrict editing button click
     * @returns {void}
     */
    private onRestrictEditingClick;
    /**
     * Update UI based on current selection
     * @returns {void}
     */
    updateSelection(): void;
    /**
     * Toggle button state
     * @param {HTMLElement} element - The HTML element to toggle
     * @param {boolean} toggle - The toggle state
     * @returns {void}
     */
    private toggleButton;
    /**
     * Destroy the ProtectGroup instance
     * @returns {void}
     */
    destroy(): void;
}
