import { DocumentEditorContainer } from '../../document-editor-container';
import { MenuEventArgs, MenuItemModel } from '@syncfusion/ej2-navigations';
/**
 * File menu constants
 */
export declare const FILE_MENU_ID: string;
export declare const NEW_DOCUMENT_ID: string;
export declare const OPEN_DOCUMENT_ID: string;
export declare const EXPORT_ID: string;
export declare const EXPORT_SFDT_ID: string;
export declare const EXPORT_DOCX_ID: string;
export declare const EXPORT_DOTX_ID: string;
export declare const EXPORT_TXT_ID: string;
export declare const PRINT_ID: string;
/**
 * FileMenu class for Document Editor Ribbon
 * @private
 */
export declare class FileMenu {
    /**
     * Container reference
     * @private
     */
    private container;
    /**
     * File picker element
     * @private
     */
    private filePicker;
    /**
     * Constructor for FileMenu class
     *
     * @param {DocumentEditorContainer} container - Document editor container reference
     */
    constructor(container: DocumentEditorContainer);
    /**
     * Get file menu items
     *
     * @returns {MenuItemModel[]} File menu items
     * @private
     */
    getFileMenuItems(): MenuItemModel[];
    /**
     * Handle file menu item selection
     *
     * @param {MenuEventArgs} args - Menu item selection arguments
     * @returns {void}
     */
    onFileMenuItemSelect(args: MenuEventArgs): void;
    /**
     * Initialize file picker
     *
     * @private
     * @returns {void}
     */
    private initializeFilePicker;
    /**
     * Handle file change event when a document is opened
     * @private
     * @returns {void}
     */
    private onFileChange;
    /**
     * Check if the file format is supported
     * @private
     * @param {string} formatType - The file format extension
     * @returns {boolean} Whether the format is supported
     */
    private isSupportedFormatType;
    /**
     * Destroy the file menu
     * @returns {void}
     */
    destroy(): void;
}
