import { DocumentEditorContainer } from '../../document-editor-container';
import { RibbonGroupModel } from '@syncfusion/ej2-ribbon';
import { RibbonGroupBase } from '../ribbon-interfaces';
export declare const LAYOUT_PARAGRAPH_GROUP: string;
export declare const INDENT_LEFT_ID: string;
export declare const INDENT_RIGHT_ID: string;
export declare const SPACING_BEFORE_ID: string;
export declare const SPACING_AFTER_ID: string;
/**
 * Layout Paragraph group implementation for simplified mode
 * @private
 */
export declare class LayoutParagraphGroup extends RibbonGroupBase {
    private indentLeftNumericBox;
    private indentRightNumericBox;
    private spacingBeforeNumericBox;
    private spacingAfterNumericBox;
    private templateContainer;
    isInitilized: boolean;
    private eventHandlers;
    /**
     * Constructor for LayoutParagraphGroup class
     *
     * @param {DocumentEditorContainer} container - DocumentEditorContainer instance
     */
    constructor(container: DocumentEditorContainer);
    /**
     * Get the Ribbon items for Layout Paragraph group
     *
     * @returns {RibbonGroupModel} The ribbon group model
     */
    getGroupModel(): RibbonGroupModel;
    /**
     * Creates indent left input template
     *
     * @returns {void}
     */
    private createIndentLeftTemplate;
    /**
     * Creates indent right input template
     *
     * @returns {void}
     */
    private createIndentRightTemplate;
    /**
     * Creates spacing before input template
     *
     * @returns {void}
     */
    private createSpacingBeforeTemplate;
    /**
     * Creates spacing after input template
     *
     * @returns {void}
     */
    private createSpacingAfterTemplate;
    /**
     * Initializes the NumericTextBox instances
     *
     * @returns {void}
     */
    initializeNumericTextBoxes(): void;
    /**
     * Resets the initialization state to allow re-initialization after layout changes
     *
     * @returns {void}
     */
    resetInitializationState(): void;
    private initializeIndentLeftNumericBox;
    private initializeIndentRightNumericBox;
    private initializeSpacingBeforeNumericBox;
    private initializeSpacingAfterNumericBox;
    private initializeNumericBox;
    private onIndentLeftChange;
    private onIndentRightChange;
    private onSpacingBeforeChange;
    private onSpacingAfterChange;
    private applyIndentLeft;
    private applyIndentRight;
    private applySpacingBefore;
    private applySpacingAfter;
    /**
     * Updates the paragraph formatting properties in the UI
     *
     * @returns {void}
     * @private
     */
    updateSelection(): void;
    /**
     * Clean up resources when destroyed
     *
     * @returns {void}
     */
    destroy(): void;
}
