import { DocumentEditorContainer } from '../../document-editor-container';
import { RibbonGroupModel } from '@syncfusion/ej2-ribbon';
export declare const SIZE_GROUP: string;
export declare const WIDTH_BOX_ID: string;
export declare const HEIGHT_BOX_ID: string;
export declare const ASPECT_RATIO_ID: string;
/**
 * Size group implementation for Picture Format tab
 * @private
 */
export declare class SizeGroup {
    private container;
    private localObj;
    private isMaintainAspectRatio;
    private widthNumericBox;
    private heightNumericBox;
    private aspectRatioBtn;
    private templateContainer;
    isInitialized: boolean;
    /**
     * Constructor for SizeGroup class
     * @param {DocumentEditorContainer} container - DocumentEditorContainer instance
     */
    constructor(container: DocumentEditorContainer);
    /**
     * Get the Ribbon items for Size group
     * @returns {RibbonGroupModel} - Ribbon group model for Size group
     * @private
     */
    getSizeGroup(): RibbonGroupModel;
    private createWidthTemplate;
    private createHeightTemplate;
    /**
     * Initializes the NumericTextBox instances
     * @returns {void}
     * @private
     */
    initializeNumericTextBoxes(): void;
    private initializeWidthNumericBox;
    private initializeHeightNumericBox;
    private onWidthChange;
    private onHeightChange;
    private onAspectRatioBtnClick;
    /**
     * Updates the size property controls with current image properties
     * @returns {void}
     * @private
     */
    updateSizeProperties(): void;
    /**
     * Applies the width value to the selected image
     * @returns {void}
     * @private
     */
    applyImageWidth(): void;
    private applyImageHeight;
    /**
     * Resets the initialization state to allow re-initialization after layout changes
     * @returns {void}
     * @private
     */
    resetInitializationState(): void;
    /**
     * Clean up resources when destroyed
     * @returns {void}
     * @private
     */
    destroy(): void;
}
