import { RibbonGroupBase } from '../ribbon-interfaces';
import { DocumentEditorContainer } from '../../document-editor-container';
import { RibbonGroupModel } from '@syncfusion/ej2-ribbon';
export declare const PROTECT_GROUP: string;
export declare const PROTECT_DOCUMENT_ID: string;
export declare const READ_ONLY_ID: string;
export declare const RESTRICT_EDITING_ID: string;
/**
 * Represents the Protect Group in Review
 * @private
 */
export declare class ProtectGroup extends RibbonGroupBase {
    /**
     * Constructor for the ProtectGroup
     * @param {DocumentEditorContainer} container - DocumentEditorContainer instance
     */
    constructor(container: DocumentEditorContainer);
    /**
     * Gets the ribbon group model for Protect
     * @returns {RibbonGroupModel} The ribbon group model
     */
    getGroupModel(): RibbonGroupModel;
    /**
     * Handle selection in protect document dropdown
     * @param {MenuEventArgs} args - Menu event arguments
     * @returns {void}
     */
    private protectDocumentDropdownHandler;
    /**
     * Handle rendering of protection dropdown items to show current state
     * @param {MenuEventArgs} args - Menu event arguments
     * @returns {void}
     */
    private onBeforeRenderProtectDropdown;
    /**
     * Updates the read-only icon in the dropdown item
     * @param {HTMLElement} menuElement - The menu element
     * @returns {void}
     */
    private updateReadOnlyIcon;
    /**
     * Toggles the selected icon state
     * @param {HTMLElement} icon - The icon element
     * @param {boolean} isSelected - Whether the item is selected
     * @returns {void}
     */
    private toggleSelectedIcon;
    /**
     * Update UI based on document protection state
     * @returns {void}
     */
    updateSelection(): void;
}
