import { DocumentEditorContainer } from '../../document-editor-container';
import { Ribbon as EJ2Ribbon, RibbonContextualTabSettingsModel } from '@syncfusion/ej2-ribbon';
/**
 * Manages contextual tabs in the ribbon
 * @private
 */
export declare class RibbonContextualTabManager {
    /**
     * @private
     */
    private container;
    private ribbonDocumentEditor;
    /**
     * Flag to track if the current operation is content change
     * @private
     */
    private isContentChange;
    /**
     * Constructor for RibbonContextualTabManager
     * @param {DocumentEditorContainer} container - Document editor container
     */
    constructor(container: DocumentEditorContainer);
    /**
     * Set content change flag
     * @param {boolean} value - Flag value
     * @returns {void}
     * @private
     */
    setContentChangeFlag(value: boolean): void;
    /**
     * Get contextual tabs configuration
     * @returns {any[]} Array of contextual tab configurations
     * @private
     */
    getContextualTabs(): RibbonContextualTabSettingsModel[];
    /**
     * Update contextual tabs visibility based on selection
     * @param {Ribbon} ribbon - Ribbon instance
     * @returns {void}
     * @private
     */
    updateContextualTabs(ribbon: EJ2Ribbon): void;
    private updateTableContextualTabs;
    /**
     * hide the contextual tab
     * @param {EJ2Ribbon} ribbon - ribbon instance
     * @returns {void}
     * @private
     */
    hideContextualTab(ribbon: EJ2Ribbon): void;
    private updateHeaderFooterContextualTab;
    private updatePictureFormatContextualTab;
}
