import { LauncherClickEventArgs, TabSelectedEventArgs } from '@syncfusion/ej2-ribbon';
import { Ribbon } from './../ribbon';
import { DocumentEditorContainer } from '../../document-editor-container';
import { MenuEventArgs } from '@syncfusion/ej2-navigations';
/**
 * Manages ribbon events
 * @private
 */
export declare class RibbonEventManager {
    /**
     * @private
     */
    private ribbon;
    private container;
    /**
     * Constructor for RibbonEventManager
     * @param {Ribbon} ribbon - Ribbon instance
     * @param {DocumentEditorContainer} container - Document editor container instance
     */
    constructor(ribbon: Ribbon, container: DocumentEditorContainer);
    /**
     * Handle launcher icon click
     * @param {LauncherClickEventArgs} args - Event arguments
     * @returns {void}
     * @private
     */
    onLauncherIconClicked(args: LauncherClickEventArgs): void;
    /**
     * Handle file menu item selection
     * @param {MenuEventArgs} args - Event arguments
     * @returns {void}
     * @private
     */
    onFileMenuItemSelect(args: MenuEventArgs): void;
    /**
     * Handle tab selection
     * @param {TabSelectedEventArgs} args - Event arguments
     * @returns {void}
     * @private
     */
    onTabSelected(args: TabSelectedEventArgs): void;
    /**
     * Handle ribbon layout change
     * @returns {void}
     * @private
     */
    onRibbonLayoutChange(): void;
    /**
     * Resize the editor container based on ribbon layout
     * @param {HTMLElement} editorContainer - The editor container element
     * @returns {void}
     * @private
     */
    private resizeEditor;
    /**
     * Update numeric text boxes based on selected tab
     * @param {string} header - The tab header text
     * @returns {void}
     * @private
     */
    private updateNumericTextBox;
}
