import { DocumentEditorContainer } from '../../document-editor-container';
import { Ribbon } from '@syncfusion/ej2-ribbon';
/**
 * Manages ribbon state
 * @private
 */
export declare class RibbonStateManager {
    /**
     * @private
     */
    private container;
    /**
     * Constructor for RibbonStateManager
     * @param {DocumentEditorContainer} container - Document editor container
     */
    constructor(container: DocumentEditorContainer);
    /**
     * Update ribbon state based on current selection
     * @param {Ribbon} ribbon - Ribbon instance
     * @returns {void}
     * @public
     */
    updateRibbonState(ribbon: Ribbon): void;
    /**
     * Enable or disable ribbon items based on protection state
     * @param {Ribbon} ribbon - Ribbon instance
     * @param {boolean} enable - Whether to enable or disable items
     * @returns {void}
     * @private
     */
    enableDisableRibbonItem(ribbon: Ribbon, enable: boolean): void;
    /**
     * Handle home tab protection
     * @param {Ribbon} ribbon - Ribbon instance
     * @param {RibbonTabModel} tab - Tab object
     * @returns {void}
     * @private
     */
    private handleHomeTabProtection;
    /**
     * Enable only find-related items
     * @param {Ribbon} ribbon - Ribbon instance
     * @param {RibbonGroupModel} group - Group object
     * @returns {void}
     * @private
     */
    private enableOnlyFindItems;
    /**
     * Handle review tab protection
     * @param {Ribbon} ribbon - Ribbon instance
     * @param {RibbonTabModel} tab - Tab object
     * @returns {void}
     * @private
     */
    private handleReviewTabProtection;
}
