import { DocumentEditorContainer } from '../../document-editor-container';
import { L10n } from '@syncfusion/ej2-base';
import { RibbonTabModel } from '@syncfusion/ej2-ribbon';
import { MenuItemModel } from '@syncfusion/ej2-navigations';
import { HomeTab } from './../home-tab/home-tab';
import { InsertTab } from './../insert-tab/insert-tab';
import { ViewTab } from './../view-tab/view-tab';
import { TableLayoutTab } from './../table-layout-tab/table-tab';
import { HeaderFooterTab } from './../header-footer-tab/header-footer-tab';
import { PictureFormatTab } from './../picture-format-tab/picture-format-tab';
import { ReferenceTab } from './../reference-tab/reference-tab';
import { LayoutTab } from './../layout-tab/layout-tab';
import { ReviewTab } from './../review-tab/review-tab';
import { TableDesignTab } from './../table-design-tab/table-design-tab';
import { FileMenu } from './../file-menu/file-menu';
import { DeveloperTab } from '../developer-tab/developer-tab';
/**
 * Manages all ribbon tabs
 * @private
 */
export declare class RibbonTabManager {
    /**
     * @private
     */
    private container;
    /**
     * @private
     */
    private localObj;
    /**
     * @private
     */
    fileMenu: FileMenu;
    /**
     * @private
     */
    homeTab: HomeTab;
    /**
     * @private
     */
    insertTab: InsertTab;
    /**
     * @private
     */
    viewTab: ViewTab;
    /**
     * @private
     */
    tableLayoutTab: TableLayoutTab;
    /**
     * @private
     */
    headerFooterTab: HeaderFooterTab;
    /**
     * @private
     */
    pictureFormatTab: PictureFormatTab;
    /**
     * @private
     */
    referenceTab: ReferenceTab;
    /**
     * @private
     */
    layoutTab: LayoutTab;
    /**
     * @private
     */
    reviewTab: ReviewTab;
    /**
     * @private
     */
    tableDesignTab: TableDesignTab;
    /**
     * @private
     */
    developerTab: DeveloperTab;
    /**
     * Constructor for RibbonTabManager
     * @param {DocumentEditorContainer} container - Document editor container
     * @param {L10n} localObj - Localization object
     */
    constructor(container: DocumentEditorContainer, localObj: L10n);
    /**
     * Initialize all tab instances
     * @returns {void}
     * @private
     */
    private initializeTabs;
    /**
     * Get all ribbon tabs
     * @returns {RibbonTabModel[]} Array of ribbon tab models
     * @private
     */
    getRibbonTabs(): RibbonTabModel[];
    /**
     * Get file menu items
     * @returns {MenuItemModel[]} Array of menu item models
     * @private
     */
    getFileMenuItems(): MenuItemModel[];
    /**
     * Update style gallery in home tab
     * @returns {void}
     * @private
     */
    updateStyleGallery(): void;
    /**
     * Update all tabs based on current selection
     * @returns {void}
     * @private
     */
    updateAllTabs(): void;
    /**
     * Get tab instance by header
     * @param {string} header - Tab header text
     * @returns {any} Tab instance
     * @private
     */
    getTabByHeader(header: string): any;
    /**
     * Destroy all tab instances
     * @returns {void}
     * @private
     */
    destroy(): void;
}
