import { DocumentEditorContainer } from '../../document-editor-container';
import { RibbonGroupModel } from '@syncfusion/ej2-ribbon';
/**
 * BordersGroup class provides the borders and shading options for table design
 * @private
 */
export declare class BordersGroup {
    private container;
    private localObj;
    private commonID;
    private borderColor;
    private borderWidth;
    private templateContainer;
    private borderDropdown;
    private widthOptions;
    /**
     * Constructor for the BordersGroup class
     * @param {DocumentEditorContainer} container - DocumentEditorContainer instance
     */
    constructor(container: DocumentEditorContainer);
    /**
     * Gets the Borders group configuration
     * @returns {RibbonGroupModel} RibbonGroupModel for the Borders group
     */
    getBordersGroup(): RibbonGroupModel;
    /**
     * Gets the Borders dropdown configuration
     * @returns {RibbonItemModel} RibbonItemModel for the Borders dropdown
     */
    private getBordersDropdown;
    /**
     * Gets the Border Color Picker configuration
     * @returns {RibbonItemModel} RibbonItemModel for the Border Color Picker
     */
    private getBorderColorPicker;
    /**
     * Gets the Border Width Dropdown configuration
     * @returns {RibbonItemModel} RibbonItemModel for the Border Width Dropdown
     */
    private getBorderWidthDropdown;
    /**
     * Creates the border size dropdown items
     * @param {HTMLElement} ulTag - The ul element to append the items to
     * @returns {void}
     */
    private createBorderSizeItems;
    /**
     * Handles border width change
     * @param {string} width - The selected border width
     * @returns {void}
     */
    private onBorderWidthChange;
    /**
     * Applies the border based on the selected option
     * @param {string} borderId - The ID of the selected border option
     * @returns {void}
     */
    private applyBorder;
    /**
     * Clean up resources when destroyed
     * @returns {void}
     */
    destroy(): void;
}
