import { DocumentEditorContainer } from '../../document-editor-container';
import { RibbonGroupModel } from '@syncfusion/ej2-ribbon';
export declare const SHADING_BUTTON_ID: string;
/**
 * ShadingGroup class provides the shading options for table design
 * @private
 */
export declare class ShadingGroup {
    private container;
    private localObj;
    private commonID;
    private colorPickerId;
    private shadingButtonId;
    private currentShadingColor;
    private colorPickerElement;
    private colorPicker;
    /**
     * Constructor for the ShadingGroup class
     * @param {DocumentEditorContainer} container - DocumentEditorContainer instance
     */
    constructor(container: DocumentEditorContainer);
    /**
     * Create the Shading group configuration
     * @returns {void}
     */
    private createColorPickerElement;
    /**
     * Gets the Shading group configuration
     * @returns {RibbonGroupModel} - The Shading group configuration
     * @private
     */
    getShadingGroup(): RibbonGroupModel;
    /**
     * Gets the Shading Split Button configuration with color picker dropdown
     * @returns {RibbonItemModel} - The Shading Split Button configuration
     * @private
     */
    private getShadingSplitButton;
    /**
     * Shows the color picker dialog
     * @returns {void}
     * @private
     */
    showColorPicker(): void;
    /**
     * Applies the shading color to the selected table or cells
     * @param {string} color - The color to apply as shading
     * @returns {void}
     */
    private applyShadingColor;
    /**
     * Updates the shading color to reflect the current selection
     * @returns {void}
     * @private
     */
    updateShadingColor(): void;
    /**
     * Clean up resources when destroyed
     * @returns {void}
     * @private
     */
    destroy(): void;
}
